/*
 * Audit VFS module.
 * Copyright (C) Anaoliy Okhotnikov, 2005-2006
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <mysql/mysql.h>
#include <mysql/errmsg.h>
#include <stdlib.h>
#include <stdio.h>
/* #include <time.h> */

#include "includes.h"
#include "my_helpers.h"

#define STRING_SIZE 256

/* #define INSERT_AUDIT2 "INSERT INTO audit2 (id,useraddr,share,op_name,err_msg,op_msg,username) VALUES(?,?,?,?,?,?,?)" */
#define INSERT_AUDIT2 "INSERT INTO audit2 (username,useraddr,share,op_name,err_msg,op_msg,cnt) VALUES('"
#define UPDATE_AUDIT2 "UPDATE audit2 SET cnt=cnt+1 WHERE "
			    
unsigned char*user_data;
unsigned char*host_data;
unsigned char*share_data;
unsigned char*op_name_data;
unsigned char*err_msg_data;
unsigned char*op_msg_data;

static pstring db_host,db_user,db_pass,db_name,pdb_port;
unsigned int  db_port = 0;

MYSQL m;
MYSQL_STMT*stmt;
int my_audit_connected = 0;
int int_data;
unsigned int cnt = 1;
unsigned long user_size,host_size,share_size,op_name_size,err_msg_size,op_msg_size;

static char *audit_prefix(connection_struct *conn);
static BOOL log_success(vfs_handle_struct *handle, vfs_op_type op);
static BOOL log_failure(vfs_handle_struct *handle, vfs_op_type op);
static const char *audit_opname(vfs_op_type op);

static pstring last_query = "";

/*************************************************************************************/

int my_audit_log(unsigned char*username,unsigned char*useraddr,unsigned char*share,
    unsigned const char*op_name,unsigned char*err_msg,unsigned char*op_msg)
{
 static pstring query;
 /* DEBUG(0,("mysql_logger.h: %s|%s|%s|%s|%s|%s \n", username, useraddr, share, op_name, err_msg, op_msg)); */
 /* check connection */
 if(my_audit_connected) {
    /* Create a buffer to write our slash-encoded data int; it must be at least twice
    the size of the input string plus one byte for the null terminator */
    user_data    = malloc(2*strlen(username)+1);
    host_data    = malloc(2*strlen(useraddr)+1);
    share_data   = malloc(2*strlen(share)+1);
    op_name_data = malloc(2*strlen(op_name)+1);
    err_msg_data = malloc(2*strlen(err_msg)+1);
    op_msg_data  = malloc(2*strlen(op_msg)+1);
    
    /* Escape any MySQL-unsafe characters */
    user_size    = mysql_real_escape_string(&m, user_data,    username, strlen(username));
    host_size    = mysql_real_escape_string(&m, host_data,    useraddr, strlen(useraddr));
    share_size   = mysql_real_escape_string(&m, share_data,   share,    strlen(share));
    op_name_size = mysql_real_escape_string(&m, op_name_data, op_name,  strlen(op_name));
    err_msg_size = mysql_real_escape_string(&m, err_msg_data, err_msg,  strlen(err_msg));
    op_msg_size  = mysql_real_escape_string(&m, op_msg_data,  op_msg,   strlen(op_msg));

    /* Construct query */
    pstrcpy(query, INSERT_AUDIT2);
    pstrcat(query, user_data);    pstrcat(query, "','");
    pstrcat(query, host_data);    pstrcat(query, "','");
    pstrcat(query, share_data);   pstrcat(query, "','");
    pstrcat(query, op_name_data); pstrcat(query, "','");
    pstrcat(query, err_msg_data); pstrcat(query, "','");
    pstrcat(query, op_msg_data);  pstrcat(query, "','");
    pstrcat(query, "1");          pstrcat(query, "')");
    
    /* Execute the INSERT statement */
	if (strcmp(query,last_query)==0) {
	  cnt++;
	  pstrcpy(query, UPDATE_AUDIT2);
	  pstrcat(query, " username='"); pstrcat(query, user_data);    pstrcat(query, "' AND ");
	  pstrcat(query, " useraddr='"); pstrcat(query, host_data);    pstrcat(query, "' AND ");
	  pstrcat(query, " share='");    pstrcat(query, share_data);   pstrcat(query, "' AND ");
	  pstrcat(query, " op_name='");  pstrcat(query, op_name_data); pstrcat(query, "' AND ");
	  pstrcat(query, " err_msg='");  pstrcat(query, err_msg_data); pstrcat(query, "' AND ");
	  pstrcat(query, " op_msg='");   pstrcat(query, op_msg_data);  pstrcat(query, "'"); 
	} else {
	  cnt = 1;
	  pstrcpy(last_query, query);
	}
    int r = mysql_real_query(&m, query, strlen(query));
    if(r) {
        /* Error */
        switch(r) {
    	    case CR_COMMANDS_OUT_OF_SYNC:
    		DEBUG(1,("mysql_logger.h: Commands were executed in an improper order. \n"));
    		break;
    	    case CR_OUT_OF_MEMORY:
    	        DEBUG(1,("mysql_logger.h: Out of memory. \n"));
    	        break;
    	    case CR_SERVER_GONE_ERROR:
    	        DEBUG(1,("mysql_logger.h: The MySQL server has gone away. \n"));
    	        break;
    	    case CR_SERVER_LOST:
    	        DEBUG(1,("mysql_logger.h: The connection to the server was lost during the query. \n"));
    	        break;
    	    case CR_UNKNOWN_ERROR:
    	        DEBUG(1,("mysql_logger.h: An unknown error occurred. \n"));
    	        break;
    	    default:
    	        DEBUG(1,("mysql_logger.h: Unknown return code: %d - %s \n", r, mysql_error(&m)));
        }
    }
    /* Free allocated memory */
    if(user_size>0)    free(user_data);
    if(host_size>0)    free(host_data);
    if(share_size>0)   free(share_data);
    if(op_name_size>0) free(op_name_data);
    if(err_msg_size>0) free(err_msg_data);
    if(op_msg_size>0)  free(op_msg_data);
 } else {
    /* Error */
    DEBUG(1,("mysql_logger.h: No database connection \n"));
    DEBUG(2,("mysql_logger.h: %s|%s|%s|%s|%s|%s \n", username, useraddr, share, op_name, err_msg, op_msg));
 }
 return 0;
}

/*************************************************************************************/

int my_audit_db_init(void)
{
 int server_argc = -1; // negative number, to contact an external server
 //mysql_library_init(server_argc, NULL, NULL);
 mysql_init(&m);
 mysql_options(&m,MYSQL_OPT_COMPRESS,0);
 //mysql_options(&m,MYSQL_OPT_RECONNECT,1);
 mysql_options(&m,MYSQL_READ_DEFAULT_GROUP,"smb_audit");
 return 0;
}

/*************************************************************************************/

int my_audit_db_connect(void)
{
 my_audit_connected = 1;
 /* Now let's try smb.conf */
 if(!db_host) pstrcpy(db_host,NULL);
 if(!db_user) pstrcpy(db_user,NULL);
 if(!db_pass) pstrcpy(db_pass,NULL);
 if(!db_name) pstrcpy(db_name,NULL);
 if (!mysql_real_connect(&m,db_host,db_user,db_pass,db_name,db_port,NULL,0)) {
  my_audit_connected = 0;
 }
 return my_audit_connected;
}

/*************************************************************************************/

int my_audit_db_close()
{
 if(my_audit_connected)
 {
  my_audit_connected = 0;
  mysql_close(&m); 
 } 
/* 
 mysql_library_end();
*/
 return 0;
}

/*************************************************************************************/

static void do_log(vfs_op_type op, BOOL success, vfs_handle_struct *handle,
		   const char *format, ...)
{
	fstring err_msg;
	pstring op_msg;
	va_list ap;

	if (success && (!log_success(handle, op)))
		return;

	if (!success && (!log_failure(handle, op)))
		return;

	if (success)
		fstrcpy(err_msg, "ok");
	else
		fstr_sprintf(err_msg, "fail (%s)", strerror(errno));

	va_start(ap, format);
	vsnprintf(op_msg, sizeof(op_msg), format, ap);
	va_end(ap);

	static pstring user,host,share;
	pstrcpy(user, "%u");
	standard_sub_conn(handle->conn, user, sizeof(user)-1);
	pstrcpy(host, "%I");
	standard_sub_conn(handle->conn, host, sizeof(host)-1);
	pstrcpy(share, "%S");
	standard_sub_conn(handle->conn, share, sizeof(share)-1);

         /* check connection */
        if(!my_audit_connected) {
	    pstrcpy(db_host,lp_parm_const_string(SNUM(handle->conn), "mysql_audit", "host", "localhost"));
	    pstrcpy(db_user,lp_parm_const_string(SNUM(handle->conn), "mysql_audit", "user", "smbd"));
	    pstrcpy(db_pass,lp_parm_const_string(SNUM(handle->conn), "mysql_audit", "pass", "smbd001"));
	    pstrcpy(db_name,lp_parm_const_string(SNUM(handle->conn), "mysql_audit", "name", "smbd"));
	    db_port = lp_parm_int(SNUM(handle->conn), "mysql_audit", "port", 3306);
	
            my_audit_db_init();
            if(my_audit_db_connect()) {
                DEBUG(1,("mysql_logger.h: DB connection SUCCESS \n"));
            } else {
		DEBUG(5,("mysql_logger.h: %s|%s|%s|%s|%d \n", db_host,db_user,db_pass,db_name,db_port));
                DEBUG(0,("mysql_logger.h: DB connection FAILED \n"));
                return;
            }
        }
	
	my_audit_log(user, host, share, audit_opname(op), err_msg, op_msg);
	
//	my_audit_log(int what,char*user, char*client_address, int service, char*message, char*status)	
//	syslog(audit_syslog_priority(handle), "%s|%s|%s|%s\n",
//	       audit_prefix(handle->conn), audit_opname(op), err_msg, op_msg);

	return;
}

/*************************************************************************************/
