/*
 * Audit VFS module.
 * Copyright (C) Anaoliy Okhotnikov, 2005-2006
 *
 * Modified by Jonis Maurin Ceara for pgsql support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <libpq-fe.h>
#include <stdlib.h>
#include <stdio.h>
/* #include <time.h> */

#include "includes.h"
#include "my_helpers.h"

#define STRING_SIZE 256 

#define INSERT_AUDIT2 "INSERT INTO audit2 (username,useraddr,share,op_name,err_msg,op_msg,cnt) VALUES('"
#define UPDATE_AUDIT2 "UPDATE audit2 SET cnt=cnt+1 WHERE "
    
unsigned char*user_data;
unsigned char*host_data;
unsigned char*share_data;
unsigned char*op_name_data;
unsigned char*err_msg_data;
unsigned char*op_msg_data;

static pstring db_host,db_user,db_pass,db_name,pdb_port;
unsigned int  db_port = 0;

PGconn *conn;
PGresult *result;
int RC, pqesc_error;
int my_audit_connected = 0;
unsigned int cnt = 1;
unsigned long user_size,host_size,share_size,op_name_size,err_msg_size,op_msg_size;

static char *audit_prefix(connection_struct *conn);
static BOOL log_success(vfs_handle_struct *handle, vfs_op_type op);
static BOOL log_failure(vfs_handle_struct *handle, vfs_op_type op);
static const char *audit_opname(vfs_op_type op);

static pstring last_query = "";

/*************************************************************************************/

int my_audit_log(unsigned char*username,unsigned char*useraddr,unsigned char*share,
    unsigned const char*op_name,unsigned char*err_msg,unsigned char*op_msg)
{
 static pstring query;
 
 /* DEBUG(0,("mysql_logger.h: %s|%s|%s|%s|%s|%s \n", username, useraddr, share, op_name, err_msg, op_msg)); */
 /* check connection */
 if(my_audit_connected) {
    /* Create a buffer to write our slash-encoded data int; it must be at least twice
    the size of the input string plus one byte for the null terminator */
    user_data    = malloc(2*strlen(username)+1);
    host_data    = malloc(2*strlen(useraddr)+1);
    share_data   = malloc(2*strlen(share)+1);
    op_name_data = malloc(2*strlen(op_name)+1);
    err_msg_data = malloc(2*strlen(err_msg)+1);
    op_msg_data  = malloc(2*strlen(op_msg)+1);

    /* Escape any PostgrsSQL-unsafe characters */
    user_size    = PQescapeStringConn (conn, user_data,    username, strlen(username), &pqesc_error);
    host_size    = PQescapeStringConn (conn, host_data,    useraddr, strlen(useraddr), &pqesc_error);
    share_size   = PQescapeStringConn (conn, share_data,   share,    strlen(share),    &pqesc_error);
    op_name_size = PQescapeStringConn (conn, op_name_data, op_name,  strlen(op_name),  &pqesc_error);
    err_msg_size = PQescapeStringConn (conn, err_msg_data, err_msg,  strlen(err_msg),  &pqesc_error);
    op_msg_size  = PQescapeStringConn (conn, op_msg_data,  op_msg,   strlen(op_msg),   &pqesc_error);

    /* Construct query */
    pstrcpy(query, INSERT_AUDIT2);
    pstrcat(query, user_data);    pstrcat(query, "','");
    pstrcat(query, host_data);    pstrcat(query, "','");
    pstrcat(query, share_data);   pstrcat(query, "','");
    pstrcat(query, op_name_data); pstrcat(query, "','");
    pstrcat(query, err_msg_data); pstrcat(query, "','");
    pstrcat(query, op_msg_data);  pstrcat(query, "','"); 
    pstrcat(query, "1");          pstrcat(query, "')"); 
 
	
    /* DEBUG(0,("1-DEBUG: %s \n", query) ); */
	if (strcmp(query,last_query)==0) {
	  cnt++;
	  pstrcpy(query, UPDATE_AUDIT2);
	  pstrcat(query, " username='"); pstrcat(query, user_data);    pstrcat(query, "' AND ");
	  pstrcat(query, " useraddr='"); pstrcat(query, host_data);    pstrcat(query, "' AND ");
	  pstrcat(query, " share='");    pstrcat(query, share_data);   pstrcat(query, "' AND ");
	  pstrcat(query, " op_name='");  pstrcat(query, op_name_data); pstrcat(query, "' AND ");
	  pstrcat(query, " err_msg='");  pstrcat(query, err_msg_data); pstrcat(query, "' AND ");
	  pstrcat(query, " op_msg='");   pstrcat(query, op_msg_data);  pstrcat(query, "'");
	} else {
	  cnt = 1;
	  pstrcpy(last_query, query);
	}
  	result = PQexec(conn, query);
  	if(!result) {
  	  /* Error */
      DEBUG(1,("pgsql_logger.h: Cannot execute query \n"));
  	} else {
  	  switch (PQresultStatus(result)) {
	    case PGRES_TUPLES_OK:
    	  RC = PQntuples(result);
	      //return RC;
		  break;
	  }
    }
    /* Free allocated memory */
    if(user_size>0)    free(user_data);
    if(host_size>0)    free(host_data);
    if(share_size>0)   free(share_data);
    if(op_name_size>0) free(op_name_data);
    if(err_msg_size>0) free(err_msg_data);
    if(op_msg_size>0)  free(op_msg_data);
 } else {
    /* Error */
    DEBUG(1,("pgsql_logger.h: No database connection \n"));
    DEBUG(2,("pgsql_logger.h: %s|%s|%s|%s|%s|%s \n", username, useraddr, share, op_name, err_msg, op_msg));
 }
 return 0;
}

/*************************************************************************************/

int my_audit_db_init(void)
{
 // Compatibilit
	return 0;
}

/*************************************************************************************/

int my_audit_db_connect(void)
{
    static pstring connection_str;
    
    if(db_host) {
	pstrcpy(connection_str,"host=");      pstrcat(connection_str,db_host);
    }
    if(db_name) {
	pstrcat(connection_str," dbname=");   pstrcat(connection_str,db_name);
    }
    if(db_user) {
	pstrcat(connection_str," user=");     pstrcat(connection_str,db_user);
    }
    if(db_pass) {
	pstrcat(connection_str," password="); pstrcat(connection_str,db_pass);
    }
    if(db_port>0) {
	pstrcpy(pdb_port,"Kill me now! Please!");
	pstrcat(connection_str," port=");     pstrcat(connection_str,my_uitoa(db_port,pdb_port,10));
    }

    conn = PQconnectdb(connection_str);		
    if (PQstatus(conn) != CONNECTION_OK) {
	my_audit_connected = 0;
    } else {
	my_audit_connected = 1;
    }

    return my_audit_connected;	
}

/*************************************************************************************/

int my_audit_db_close()
{
 if(my_audit_connected)
 {
  my_audit_connected = 0;
  PQreset(conn);
 } 
 return 0;
}

/*************************************************************************************/

static void do_log(vfs_op_type op, BOOL success, vfs_handle_struct *handle,
		   const char *format, ...)
{
	fstring err_msg;
	pstring op_msg;
	va_list ap;

	if (success && (!log_success(handle, op)))
		return;

	if (!success && (!log_failure(handle, op)))
		return;

	if (success)
		fstrcpy(err_msg, "ok");
	else
		fstr_sprintf(err_msg, "fail (%s)", strerror(errno));

	va_start(ap, format);
	vsnprintf(op_msg, sizeof(op_msg), format, ap);
	va_end(ap);

	static pstring user,host,share;
	pstrcpy(user, "%u");
	standard_sub_conn(handle->conn, user, sizeof(user)-1);
	pstrcpy(host, "%I");
	standard_sub_conn(handle->conn, host, sizeof(host)-1);
	pstrcpy(share, "%S");
	standard_sub_conn(handle->conn, share, sizeof(share)-1);

	 /* check connection */
	if(!my_audit_connected) {
	    pstrcpy(db_host,lp_parm_const_string(SNUM(handle->conn), "pgsql_audit", "host", "localhost"));
	    pstrcpy(db_user,lp_parm_const_string(SNUM(handle->conn), "pgsql_audit", "user", "smbd"));
	    pstrcpy(db_pass,lp_parm_const_string(SNUM(handle->conn), "pgsql_audit", "pass", "smbd001"));
	    pstrcpy(db_name,lp_parm_const_string(SNUM(handle->conn), "pgsql_audit", "name", "smbd"));
	    db_port = lp_parm_int(SNUM(handle->conn), "pgsql_audit", "port", 5432);
	
	    my_audit_db_init();
	    if(my_audit_db_connect()) {
		DEBUG(1,("pgsql_logger.h: DB connection SUCCESS \n")); 
	    } else {
		DEBUG(5,("pgsql_logger.h: %s|%s|%s|%s|%d \n", db_host,db_user,db_pass,db_name,db_port));
		DEBUG(0,("pgsql_logger.h: DB connection FAILED \n")); 
		return;
	    }
	} 
	
	my_audit_log(user, host, share, audit_opname(op), err_msg, op_msg);
			
/*	syslog(audit_syslog_priority(handle), "%s|%s|%s|%s\n",
	       audit_prefix(handle->conn), audit_opname(op), err_msg, op_msg); */

	return;
}

/*************************************************************************************/

